/* -----------------------------------------------------------------------------------------------
	Shared Inverse Affine Transformation
----------------------------------------------------------------------------------------------- */ 

module aft_inv_shared(
	input i_enable,
	input [7:0] i_data_1,
	input [7:0] i_data_2,
	input [7:0] i_data_3,
	input [7:0] i_data_4,
	output [7:0] o_data_1,
	output [7:0] o_data_2,
	output [7:0] o_data_3,
	output [7:0] o_data_4
);

logic [7:0]data_1_t;
logic [7:0]data_2_t;
logic [7:0]data_3_t;
logic [7:0]data_4_t;

assign o_data_1 = data_1_t;
assign o_data_2 = data_2_t;
assign o_data_3 = data_3_t;
assign o_data_4 = data_4_t;

always_comb 
	if (i_enable) begin
		// Share 1
		data_1_t[7] = i_data_1[6]^i_data_1[4]^i_data_1[1];	
		data_1_t[6] = i_data_1[5]^i_data_1[3]^i_data_1[0];	
		data_1_t[5] = i_data_1[7]^i_data_1[4]^i_data_1[2];	
		data_1_t[4] = i_data_1[6]^i_data_1[3]^i_data_1[1];	
		data_1_t[3] = i_data_1[5]^i_data_1[2]^i_data_1[0];	
		data_1_t[2] = i_data_1[7]^i_data_1[4]^i_data_1[1]^1'b1;	
		data_1_t[1] = i_data_1[6]^i_data_1[3]^i_data_1[0];	
		data_1_t[0] = i_data_1[7]^i_data_1[5]^i_data_1[2]^1'b1;	
		// Share 2
		data_2_t[7] = i_data_2[6]^i_data_2[4]^i_data_2[1];
		data_2_t[6] = i_data_2[5]^i_data_2[3]^i_data_2[0];
		data_2_t[5] = i_data_2[7]^i_data_2[4]^i_data_2[2];
		data_2_t[4] = i_data_2[6]^i_data_2[3]^i_data_2[1];
		data_2_t[3] = i_data_2[5]^i_data_2[2]^i_data_2[0];
		data_2_t[2] = i_data_2[7]^i_data_2[4]^i_data_2[1];
		data_2_t[1] = i_data_2[6]^i_data_2[3]^i_data_2[0];
		data_2_t[0] = i_data_2[7]^i_data_2[5]^i_data_2[2];
		// Share 3
		data_3_t[7] = i_data_3[6]^i_data_3[4]^i_data_3[1];
		data_3_t[6] = i_data_3[5]^i_data_3[3]^i_data_3[0];
		data_3_t[5] = i_data_3[7]^i_data_3[4]^i_data_3[2];
		data_3_t[4] = i_data_3[6]^i_data_3[3]^i_data_3[1];
		data_3_t[3] = i_data_3[5]^i_data_3[2]^i_data_3[0];
		data_3_t[2] = i_data_3[7]^i_data_3[4]^i_data_3[1];
		data_3_t[1] = i_data_3[6]^i_data_3[3]^i_data_3[0];
		data_3_t[0] = i_data_3[7]^i_data_3[5]^i_data_3[2];
		// Share 4
		data_4_t[7] = i_data_4[6]^i_data_4[4]^i_data_4[1];
		data_4_t[6] = i_data_4[5]^i_data_4[3]^i_data_4[0];
		data_4_t[5] = i_data_4[7]^i_data_4[4]^i_data_4[2];
		data_4_t[4] = i_data_4[6]^i_data_4[3]^i_data_4[1];
		data_4_t[3] = i_data_4[5]^i_data_4[2]^i_data_4[0];
		data_4_t[2] = i_data_4[7]^i_data_4[4]^i_data_4[1];
		data_4_t[1] = i_data_4[6]^i_data_4[3]^i_data_4[0];
		data_4_t[0] = i_data_4[7]^i_data_4[5]^i_data_4[2];
	end
	else begin
		data_1_t = i_data_1;
		data_2_t = i_data_2;
		data_3_t = i_data_3;
		data_4_t = i_data_4;
	end
endmodule
