/* -----------------------------------------------------------------------------------------------
	This testbench iterates through given input file and verifies that the input matches the 
	expected response. The given txt-file contains iterations through all bits of input
	and key and checks for correct response in the output ciphertext.
----------------------------------------------------------------------------------------------- */ 

`define NULL 0

module combined_norm_tb();

logic clk;
logic encrypt;
logic rst;
logic ciphertxt_rdy;
logic plaintxt_rdy;
logic cbc_enable;
logic [0:3][0:3][7:0] plaintxt;
logic [0:15][0:4][3:0] rnd_bits;
logic [0:3][0:3][7:0] ciphertext;
logic [0:3][0:3][7:0] sm1;
logic [0:3][0:3][7:0] sm2;
logic [0:3][0:3][7:0] sm3;
logic [0:3][0:3][7:0] km1;
logic [0:3][0:3][7:0] km2;
logic [255:0] key;
logic [1:0] key_rdy;
logic key_ack;

logic [0:3][0:3][7:0] iv;
logic [0:3][0:3][7:0] state_reg;
logic [0:3][0:3][7:0] sbox_output;
logic [0:3][0:3][7:0] add_roundkey_output;

integer error_count;

assign state_reg = aes.share_1.o_data ^ aes.share_2.o_data ^ aes.share_3.o_data;
assign add_roundkey_output = aes.ci.sm_1_xor ^ aes.ci.sm_2_xor ^ aes.ci.sm_3_xor;
assign sbox_output = aes.sb.o_data1 ^ aes.sb.o_data2 ^ aes.sb.o_data3;

aes_core_shared_norm aes(
	.i_clk(clk),
	.i_rst_n(rst),
	.i_encrypt(encrypt), 		// flag to set encrypt or decrypt mode
	.i_data(plaintxt),
	.i_data_rdy(plaintxt_rdy),
	.i_cbc_enable(cbc_enable),

	.i_sm_1(sm1),
	.i_sm_2(sm2),
	.i_sm_3(sm3),
	.i_km_1(km1),
	.i_km_2(km2),

	.i_iv(iv),

	.i_key(key),
	.i_key_rdy(key_rdy),
	.i_rm_bits(rnd_bits),
	.o_key_ack(key_ack),
	.o_data_rdy(ciphertxt_rdy),
	.o_data(ciphertext)
);

int count = 10000;
class random_input;
	rand bit [127:0] plaintxt;
	rand bit [127:0] s1;
	rand bit [127:0] s2;
	rand bit [127:0] s3;
	rand bit [127:0] k1;
	rand bit [127:0] k2;
	rand bit [0:15][0:4][3:0] rbits;
endclass
static random_input idata = new();

integer fp, c, r;
string l;
logic [2:0] mode_test;
logic [127:0] result;

always #1 clk = !clk;
always #1 begin
	idata.randomize();
	sm1 = idata.s1;
	sm2 = idata.s2;
	sm3 = idata.s3;
	km1 = idata.k1;
	km2 = idata.k2;
	rnd_bits = idata.rbits;
end

initial 
begin: run_vector_test
	error_count = 0;
	clk = 0;
	rst = 0;
	cbc_enable = 1'b0;
	result = '{default: 0};
	plaintxt_rdy = 1'b0;
	key = '{default: 0};
	encrypt = 1'b1;
	plaintxt = '{default: 0};
	key_rdy = 2'b00;
	iv = '{default: 0};
	rst = 1'b0;
	#4
	rst = 1'b1;
	#4
	fp = $fopen("tv.txt", "r");
	if (fp == `NULL) 
		disable run_vector_test;
	while (!$feof(fp))
	begin
		#6
		c = $fgetc(fp);
		if (c == "/") begin
			c = $fgetc(fp);
			c = $fgetc(fp);
			case (c)
			"X": begin 
				$display("Setting 128 bit vartxt");
				key_rdy = 2'b01;
				key = '{default: 0};
				mode_test = 1;
				end
			"Y": begin 
				$display("Setting 192 bit vartxt");
				key_rdy = 2'b10;
				key = '{default: 0};
				mode_test = 2;
			end
			"Z": begin 
				$display("Setting 256 bit vartxt");
				key_rdy = 2'b11;
				key = '{default: 0};
				mode_test = 3;
			end			
			"U": begin 
				$display("Setting 128 bit varkey");
				mode_test = 4;
				plaintxt = '{default: 0};
				end
			"V": begin 
				$display("Setting 192 bit varkey");
				mode_test = 5;
				plaintxt = '{default: 0};
				end
			"W": begin 
				$display("Setting 256 bit varkey");
				mode_test = 6;
				plaintxt = '{default: 0};
				end
			endcase
			r = $ungetc(c, fp);
			r = $fgets(l, fp);
		end
		else begin
			r = $ungetc(c, fp);
			r = $fgets(l, fp);
			case(mode_test)
				1: begin 
					r = $sscanf(l, "%x %x", plaintxt, result);
				end
				2: begin 
					r = $sscanf(l, "%x %x", plaintxt, result);
				end
				3: begin 
					r = $sscanf(l, "%x %x", plaintxt, result);
				end
				4: begin 
					r = $sscanf(l, "%x %x", key, result);
					key_rdy = 2'b01;
				end
				5: begin 
					r = $sscanf(l, "%x %x", key, result);
					key_rdy = 2'b10;
				end
				6: begin 
					r = $sscanf(l, "%x %x", key, result);
					key_rdy = 2'b11;
				end
			endcase
			#2	
			if (key_rdy != 2'b00) begin
				key_rdy = 2'b00;
				wait(!key_ack);
			end
			wait(key_ack)
			if (c != 32'hFFFF_FFFF) begin
				#3
				plaintxt_rdy = 1'b1;
				#6 
				plaintxt_rdy = 1'b0;
				wait(ciphertxt_rdy);
			end
		end
	end
	$display("Number of errors detected in encryption: %d", error_count);
	$fclose(fp);
	encrypt = 1'b0;
	fp = $fopen("tv.txt", "r");
	if (fp == `NULL)
		disable run_vector_test;
	while (!$feof(fp))
	begin
		#6
		c = $fgetc(fp);
		if (c == "/") begin
			c = $fgetc(fp);
			c = $fgetc(fp);
			case (c)
			"X": begin 
				$display("Setting 128 bit vartxt");
				key_rdy = 2'b01;
				key = '{default: 0};
				mode_test = 1;
				end
			"Y": begin 
				$display("Setting 192 bit vartxt");
				key_rdy = 2'b10;
				key = '{default: 0};
				mode_test = 2;
			end
			"Z": begin 
				$display("Setting 256 bit vartxt");
				key_rdy = 2'b11;
				key = '{default: 0};
				mode_test = 3;
			end			
			"U": begin 
				$display("Setting 128 bit varkey");
				mode_test = 4;
				result = '{default: 0};
				end
			"V": begin 
				$display("Setting 192 bit varkey");
				mode_test = 5;
				result = '{default: 0};
				end
			"W": begin 
				$display("Setting 256 bit varkey");
				mode_test = 6;
				result = '{default: 0};
				end
			endcase
			r = $ungetc(c, fp);
			r = $fgets(l, fp);
		end
		else begin
			r = $ungetc(c, fp);
			r = $fgets(l, fp);
			case(mode_test)
				1: begin 
					r = $sscanf(l, "%x %x", result, plaintxt);
				end
				2: begin 
					r = $sscanf(l, "%x %x", result, plaintxt);
				end
				3: begin 
					r = $sscanf(l, "%x %x", result, plaintxt);
				end
				4: begin 
					r = $sscanf(l, "%x %x", key, plaintxt);
					key_rdy = 2'b01;
				end
				5: begin 
					r = $sscanf(l, "%x %x", key, plaintxt);
					key_rdy = 2'b10;
				end
				6: begin 
					r = $sscanf(l, "%x %x", key, plaintxt);
					key_rdy = 2'b11;
				end
			endcase
			#2	
			if (key_rdy != 2'b00) begin
				key_rdy = 2'b00;
				wait(!key_ack);
			end
			wait(key_ack)
			if (c != 32'hFFFF_FFFF) begin
				#3
				plaintxt_rdy = 1'b1;
				#6 
				plaintxt_rdy = 1'b0;
				wait(ciphertxt_rdy);
			end
	end
end
	$display("Number of errors detected in decryption: %d", error_count);
	$fclose(fp);
	$stop;
end

o1: assert property (@(posedge clk) ciphertxt_rdy |-> ({>>{ciphertext}} == result))
	else $error("Error #%d, \nciphertext:%h\nresult: %h", ++error_count, {>>{ciphertext}}, result);
c1: cover property (@(posedge clk) ciphertxt_rdy);

endmodule
