`define NULL 0

module combined_simple_poly_tb();

logic clk;
logic encrypt;
logic rst;
logic ciphertxt_rdy;
logic plaintxt_rdy;
logic cbc_enable;
logic [0:3][0:3][7:0] plaintxt;
logic [0:3][0:3][7:0] ciphertext;
logic [255:0] key;
logic [1:0] key_rdy;
logic key_ack;

logic [0:3][0:3][7:0] iv;
integer error_count;


aes_core_um_poly aes(
	.i_clk(clk),
	.i_rst_n(rst),
	.i_encrypt(encrypt), 		// flag to set encrypt or decrypt mode
	.i_data(plaintxt),
	.i_data_rdy(plaintxt_rdy),
	.i_cbc_enable(cbc_enable),

	.i_iv(iv),

	.i_key(key),
	.i_key_rdy(key_rdy),
	.o_key_ack(key_ack),
	.o_data_rdy(ciphertxt_rdy),
	.o_data(ciphertext)
);

integer fp, c, r;
string l;
logic [2:0] mode_test;
logic [127:0] result;

always #1 clk = !clk;

initial 
begin: run_vector_test
	error_count = 0;
	clk = 0;
	rst = 0;
	cbc_enable = 1'b0;
	result = '{default: 0};
	plaintxt_rdy = 1'b0;
	key = '{default: 0};
	encrypt = 1'b1;
	plaintxt = '{default: 0};
	key_rdy = 2'b00;
	iv = '{default: 0};
	rst = 1'b0;
	#4
	rst = 1'b1;
	#4
	fp = $fopen("tv.txt", "r");
	if (fp == `NULL) 
		disable run_vector_test;
	while (!$feof(fp))
	begin
		#6
		c = $fgetc(fp);
		if (c == "/") begin
			c = $fgetc(fp);
			c = $fgetc(fp);
			case (c)
			"X": begin 
				$display("Setting 128 bit vartxt");
				key_rdy = 2'b01;
				key = '{default: 0};
				mode_test = 1;
				end
			"Y": begin 
				$display("Setting 192 bit vartxt");
				key_rdy = 2'b10;
				key = '{default: 0};
				mode_test = 2;
			end
			"Z": begin 
				$display("Setting 256 bit vartxt");
				key_rdy = 2'b11;
				key = '{default: 0};
				mode_test = 3;
			end			
			"U": begin 
				$display("Setting 128 bit varkey");
				mode_test = 4;
				plaintxt = '{default: 0};
				end
			"V": begin 
				$display("Setting 192 bit varkey");
				mode_test = 5;
				plaintxt = '{default: 0};
				end
			"W": begin 
				$display("Setting 256 bit varkey");
				mode_test = 6;
				plaintxt = '{default: 0};
				end
			endcase
			r = $ungetc(c, fp);
			r = $fgets(l, fp);
		end
		else begin
			r = $ungetc(c, fp);
			r = $fgets(l, fp);
			case(mode_test)
				1: begin 
					r = $sscanf(l, "%x %x", plaintxt, result);
				end
				2: begin 
					r = $sscanf(l, "%x %x", plaintxt, result);
				end
				3: begin 
					r = $sscanf(l, "%x %x", plaintxt, result);
				end
				4: begin 
					r = $sscanf(l, "%x %x", key, result);
					key_rdy = 2'b01;
				end
				5: begin 
					r = $sscanf(l, "%x %x", key, result);
					key_rdy = 2'b10;
				end
				6: begin 
					r = $sscanf(l, "%x %x", key, result);
					key_rdy = 2'b11;
				end
			endcase
			#2
			if (key_rdy != 2'b00) begin
				key_rdy = 2'b00;
				wait(!key_ack);
			end
			wait(key_ack);
			if (c != 32'hFFFF_FFFF) begin
				#3
				plaintxt_rdy = 1'b1;
				#6 
				plaintxt_rdy = 1'b0;
				wait(ciphertxt_rdy);
			end
		end
	end
	$display("Number of errors detected in encryption: %d", error_count);
	$fclose(fp);
	encrypt = 1'b0;
	fp = $fopen("tv.txt", "r");
	if (fp == `NULL)
		disable run_vector_test;
	while (!$feof(fp))
	begin
		#6
		c = $fgetc(fp);
		if (c == "/") begin
			c = $fgetc(fp);
			c = $fgetc(fp);
			case (c)
			"X": begin 
				$display("Setting 128 bit vartxt");
				key_rdy = 2'b01;
				key = '{default: 0};
				mode_test = 1;
				end
			"Y": begin 
				$display("Setting 192 bit vartxt");
				key_rdy = 2'b10;
				key = '{default: 0};
				mode_test = 2;
			end
			"Z": begin 
				$display("Setting 256 bit vartxt");
				key_rdy = 2'b11;
				key = '{default: 0};
				mode_test = 3;
			end			
			"U": begin 
				$display("Setting 128 bit varkey");
				mode_test = 4;
				result = '{default: 0};
				end
			"V": begin 
				$display("Setting 192 bit varkey");
				mode_test = 5;
				result = '{default: 0};
				end
			"W": begin 
				$display("Setting 256 bit varkey");
				mode_test = 6;
				result = '{default: 0};
				end
			endcase
			r = $ungetc(c, fp);
			r = $fgets(l, fp);
		end
		else begin
			r = $ungetc(c, fp);
			r = $fgets(l, fp);
			case(mode_test)
				1: begin 
					r = $sscanf(l, "%x %x", result, plaintxt);
				end
				2: begin 
					r = $sscanf(l, "%x %x", result, plaintxt);
				end
				3: begin 
					r = $sscanf(l, "%x %x", result, plaintxt);
				end
				4: begin 
					r = $sscanf(l, "%x %x", key, plaintxt);
					key_rdy = 2'b01;
				end
				5: begin 
					r = $sscanf(l, "%x %x", key, plaintxt);
					key_rdy = 2'b10;
				end
				6: begin 
					r = $sscanf(l, "%x %x", key, plaintxt);
					key_rdy = 2'b11;
				end
			endcase
			#2	
			if (key_rdy != 2'b00) begin
				key_rdy = 2'b00;
				wait(!key_ack);
			end
			wait(key_ack)
			if (c != 32'hFFFF_FFFF) begin
				#3
				plaintxt_rdy = 1'b1;
				#6 
				plaintxt_rdy = 1'b0;
				wait(ciphertxt_rdy);
			end
	end
end
	$display("Number of errors detected in decryption: %d", error_count);
	$fclose(fp);
	$stop;
end

o1: assert property (@(posedge clk) ciphertxt_rdy |-> ({>>{ciphertext}} == result))
	else $error("Error #%d, \nciphertext:%h\nresult: %h", ++error_count, {>>{ciphertext}}, result);
c1: cover property (@(posedge clk) ciphertxt_rdy);


endmodule
