/* -----------------------------------------------------------------------------------------------
	Main Control FSM
----------------------------------------------------------------------------------------------- */ 

module control_main_fsm(
	input i_clk,
	input i_sreg_clk,
	input i_rst_n,
	input [1:0] i_key_rdy,
	input i_key_stable,
	input i_plaintxt_rdy,
	input i_lrnd,
	output [1:0] o_key_mode,
	output o_do_rounds,
	output o_wait_data
);

enum logic [2:0] {WAIT_KEY 	= 3'b001,
		  WAIT_DATA 	= 3'b010,
		  DO_ROUNDS	= 3'b100
		} current_state, next_state;

logic [1:0] mode;
logic clk;

assign o_wait_data = current_state[1];
assign o_key_mode = mode;
assign o_do_rounds = current_state[2];

always_ff @(posedge i_clk or negedge i_rst_n)
if (!i_rst_n) begin
		current_state <= WAIT_KEY;
		mode <= 0;	
	end
	else begin
		current_state <= next_state;
		if (i_key_rdy)
			mode <= i_key_rdy;
		else
			mode <= 0;
	end

always_comb
begin: state_machine
	next_state = current_state;
	unique case (current_state)
		WAIT_KEY: 	if (i_key_rdy && i_key_stable) 			next_state = WAIT_DATA;
				else						next_state = WAIT_KEY;
		WAIT_DATA:	if (i_plaintxt_rdy && i_key_stable) 		next_state = DO_ROUNDS;
				else if (!i_plaintxt_rdy && i_key_stable)  	next_state = WAIT_DATA;  
				else						next_state = WAIT_KEY;
		DO_ROUNDS:	if (i_key_stable && (!i_lrnd))			next_state = DO_ROUNDS;
				else if (i_key_stable && i_lrnd)	   	next_state = WAIT_DATA;
				else 						next_state = WAIT_KEY;
		default: 							next_state = WAIT_KEY;
	endcase
end: state_machine

endmodule 
