/* -----------------------------------------------------------------------------------------------
	Shared Core interconnect
----------------------------------------------------------------------------------------------- */ 

module core_interconnect_shared(
	input [0:3][0:3][7:0] i_data,	// plaintext, from outside
	input [0:3][0:3][7:0] i_sm_1,	// state-mask 1
	input [0:3][0:3][7:0] i_sm_2,	// state-mask 2
	input [0:3][0:3][7:0] i_sm_3,	// state-mask 3 (for use in sbox only)
	input [0:3][0:3][7:0] i_m1_key,	// output from masked key-reg 1
	input [0:3][0:3][7:0] i_m2_key,	// output from masked key-reg 2
	input [0:3][0:3][7:0] i_m3_key,	// output from masked key-reg 3
	input [0:3][0:3][7:0] i_s1_data,	// output from masked state-reg 1
	input [0:3][0:3][7:0] i_s2_data,	// output from masked state-reg 2
	input [0:3][0:3][7:0] i_s3_data,	// output from masked state-reg 3
	input [0:15][0:4][3:0] i_remask_bits,	// bits for sbox-remasking registers
	input [1:0] i_flf,		

	input [0:3][0:3][7:0] i_cbc_encrypt,
	input [0:3][0:3][7:0] i_cbc_decrypt,

	output logic [0:3][0:3][7:0] o_sb1,
	output logic [0:3][0:3][7:0] o_sb2,
	output logic [0:3][0:3][7:0] o_sb3,
	output logic [0:3][0:3][7:0] o_sb4,
	output logic [0:15][0:4][3:0] o_remask_bits,
	output logic [0:3][0:3][7:0] o_ciphertext
);

logic [0:3][0:3][7:0] sm_1_mux;
logic [0:3][0:3][7:0] sm_2_mux;
logic [0:3][0:3][7:0] sm_3_mux;

logic [0:3][0:3][7:0] sm_1_xor;
logic [0:3][0:3][7:0] sm_2_xor;
logic [0:3][0:3][7:0] sm_3_xor;

always_comb begin
	unique if ((i_flf[0] && !i_flf[1])) begin
		o_sb1 = i_data ^ i_sm_1 ^ i_sm_2 ^ i_m1_key;
		o_sb2 = 	 i_sm_1 ^ i_sm_3 ^ i_m2_key;
		o_sb3 = 	 i_sm_2  ^	   i_m3_key ^ i_cbc_encrypt;
	end
	else begin
		o_sb1 = i_s1_data ^ i_m1_key;
		o_sb2 = i_s2_data ^ i_m2_key ^ i_sm_3;
		o_sb3 = i_s3_data ^ i_m3_key;
	end
	o_sb4 = i_sm_3;
	o_remask_bits = i_remask_bits;
	unique case (i_flf[1])
		1'b1: o_ciphertext = i_s1_data ^ i_s2_data ^ i_s3_data ^ i_m1_key ^ i_m2_key ^ i_m3_key ^ i_cbc_decrypt;
		default: o_ciphertext = '{default: 0};
	endcase
end
/*

assign sm_1_xor = sm_1_mux ^ i_m1_key;
assign sm_2_xor = sm_2_mux ^ i_m2_key;
assign sm_3_xor = sm_3_mux ^ i_m3_key;

assign o_sb1 = sm_1_xor;
assign o_sb2 = sm_2_xor;
assign o_sb3 = sm_3_xor ^ i_sm_3;
assign o_sb4 = i_sm_3;
assign o_remask_bits = i_remask_bits;

always_comb begin
	unique case (i_flf[0] && !i_flf[1])
		1'b1: begin
			sm_1_mux = i_data ^ i_sm_1 ^ i_sm_2;
			sm_2_mux = i_sm_1;
			sm_3_mux = i_sm_2 ^ i_cbc_encrypt;
		end
		default: begin
			sm_1_mux = i_s1_data;
			sm_2_mux = i_s2_data;
			sm_3_mux = i_s3_data;
		end
	endcase
end

always_comb begin  
	unique case (i_flf[1])
		1'b1: o_ciphertext = sm_1_xor ^ sm_2_xor ^ sm_3_xor ^ i_cbc_decrypt;
		default: o_ciphertext = '{default: 0};
	endcase
end
*/
endmodule

