/* -----------------------------------------------------------------------------------------------
	Non-shared core interconnect
----------------------------------------------------------------------------------------------- */ 

module core_interconnect_um(
	input [0:3][0:3][7:0] i_data,	// plaintext, from outside
	input [0:3][0:3][7:0] i_m1_key,	// output from masked key-reg 1
	input [0:3][0:3][7:0] i_s1_data,	// output from masked state-reg 1
	input [1:0] i_flf,		

	input [0:3][0:3][7:0] i_cbc_encrypt,
	input [0:3][0:3][7:0] i_cbc_decrypt,

	output [0:3][0:3][7:0] o_sb,
	output logic [0:3][0:3][7:0] o_ciphertext
);

logic [0:3][0:3][7:0] sm_1_mux;

logic [0:3][0:3][7:0] sm_1_xor;

assign sm_1_xor = sm_1_mux ^ i_m1_key;

assign o_sb = sm_1_xor;


always_comb begin
	unique case (i_flf[0] && !i_flf[1])
		1'b1: begin
			sm_1_mux = i_data ^ i_cbc_encrypt;
		end
		default: begin
			sm_1_mux = i_s1_data;
		end
	endcase
end

always_comb begin  
	unique case (i_flf[1])
		1'b1: o_ciphertext = sm_1_xor ^ i_cbc_decrypt;
		default: o_ciphertext = '{default: 0};
	endcase
end
endmodule

