// Shared multiplier - takes 8 inputs of 4 bits (4x1 bit input shared 4 times), 
// outputs 4x1 bit shared 3 times = 4x3 bits = 12 bits

module gf24mul_shared_poly(

	input [3:0] i_data_1,
	input [3:0] i_data_2,
	input [3:0] i_data_3,
	input [3:0] i_data_4,

	input [3:0] i_data_5,
	input [3:0] i_data_6,
	input [3:0] i_data_7,
	input [3:0] i_data_8,

	output [3:0] o_data_1,
	output [3:0] o_data_2,
	output [3:0] o_data_3
);

logic [3:0] a;
logic [3:0] b;
logic [3:0] c;
logic [3:0] d;
logic [3:0] x;
logic [3:0] y;
logic [3:0] z;
logic [3:0] v;

logic  [2:0] o_m;
logic  [2:0] o_n;
logic  [2:0] o_o;
logic  [2:0] o_p;

// change wiring to match inputs and outputs

assign a = '{i_data_1[3], i_data_2[3], i_data_3[3], i_data_4[3]};
assign b = '{i_data_1[2], i_data_2[2], i_data_3[2], i_data_4[2]};
assign c = '{i_data_1[1], i_data_2[1], i_data_3[1], i_data_4[1]};
assign d = '{i_data_1[0], i_data_2[0], i_data_3[0], i_data_4[0]};

assign x = '{i_data_5[3], i_data_6[3], i_data_7[3], i_data_8[3]};
assign y = '{i_data_5[2], i_data_6[2], i_data_7[2], i_data_8[2]};
assign z = '{i_data_5[1], i_data_6[1], i_data_7[1], i_data_8[1]};
assign v = '{i_data_5[0], i_data_6[0], i_data_7[0], i_data_8[0]};

assign o_data_3 = '{o_m[0], o_n[0], o_o[0], o_p[0]};
assign o_data_2 = '{o_m[1], o_n[1], o_o[1], o_p[1]};
assign o_data_1 = '{o_m[2], o_n[2], o_o[2], o_p[2]};

assign o_m[0] = ((a[1] ^ a[2] ^ a[3]) && (x[1] ^ x[2])) ^ x[3] ^ 
		((a[1] ^ a[2] ^ a[3]) && (y[1] ^ y[2])) ^ y[3] ^ 
		((a[1] ^ a[2] ^ a[3]) && (z[1] ^ z[2])) ^ z[3] ^ 
		((a[1] ^ a[2] ^ a[3]) && (v[1] ^ v[2])) ^ v[3] ^ 
		((b[1] ^ b[2] ^ b[3]) && (x[1] ^ x[2])) ^ x[3] ^ 
		((b[1] ^ b[2] ^ b[3]) && (z[1] ^ z[2])) ^ z[3] ^ 
		((c[1] ^ c[2] ^ c[3]) && (x[1] ^ x[2])) ^ x[3] ^ 
		((c[1] ^ c[2] ^ c[3]) && (y[1] ^ y[2])) ^ y[3] ^ 
		((d[1] ^ d[2] ^ d[3]) && (x[1] ^ x[2])) ^ x[3];

assign o_m[1] =	((a[0] ^ a[2]) && (x[0] ^ x[3])) ^ (a[0] && x[2]) ^ a[3] ^
		((a[0] ^ a[2]) && (y[0] ^ y[3])) ^ (a[0] && y[2]) ^ a[3] ^
		((a[0] ^ a[2]) && (z[0] ^ z[3])) ^ (a[0] && z[2]) ^ a[3] ^
		((a[0] ^ a[2]) && (v[0] ^ v[3])) ^ (a[0] && v[2]) ^ a[3] ^
		((b[0] ^ b[2]) && (x[0] ^ x[3])) ^ (b[0] && x[2]) ^ b[3] ^
		((b[0] ^ b[2]) && (z[0] ^ z[3])) ^ (b[0] && z[2]) ^ b[3] ^
		((c[0] ^ c[2]) && (x[0] ^ x[3])) ^ (c[0] && x[2]) ^ c[3] ^
		((c[0] ^ c[2]) && (y[0] ^ y[3])) ^ (c[0] && y[2]) ^ c[3] ^
		((d[0] ^ d[2]) && (x[0] ^ x[3])) ^ (d[0] && x[2]) ^ d[3];

assign o_m[2] = ((a[1] ^ a[3]) && (x[0] ^ x[3])) ^ (a[0] && x[1]) ^ a[3] ^ x[3] ^
		((a[1] ^ a[3]) && (y[0] ^ y[3])) ^ (a[0] && y[1]) ^ a[3] ^ y[3] ^
		((a[1] ^ a[3]) && (z[0] ^ z[3])) ^ (a[0] && z[1]) ^ a[3] ^ z[3] ^
		((a[1] ^ a[3]) && (v[0] ^ v[3])) ^ (a[0] && v[1]) ^ a[3] ^ v[3] ^
		((b[1] ^ b[3]) && (x[0] ^ x[3])) ^ (b[0] && x[1]) ^ b[3] ^ x[3] ^
		((b[1] ^ b[3]) && (z[0] ^ z[3])) ^ (b[0] && z[1]) ^ b[3] ^ z[3] ^
		((c[1] ^ c[3]) && (x[0] ^ x[3])) ^ (c[0] && x[1]) ^ c[3] ^ x[3] ^
		((c[1] ^ c[3]) && (y[0] ^ y[3])) ^ (c[0] && y[1]) ^ c[3] ^ y[3] ^
		((d[1] ^ d[3]) && (x[0] ^ x[3])) ^ (d[0] && x[1]) ^ d[3] ^ x[3];

assign o_n[0] =	((a[1] ^ a[2] ^ a[3]) && (x[1] ^ x[2])) ^ x[3] ^ 
		((a[1] ^ a[2] ^ a[3]) && (z[1] ^ z[2])) ^ z[3] ^ 
		((b[1] ^ b[2] ^ b[3]) && (y[1] ^ y[2])) ^ y[3] ^ 
		((b[1] ^ b[2] ^ b[3]) && (v[1] ^ v[2])) ^ v[3] ^ 
		((c[1] ^ c[2] ^ c[3]) && (x[1] ^ x[2])) ^ x[3] ^ 
		((d[1] ^ d[2] ^ d[3]) && (y[1] ^ y[2])) ^ y[3];

assign o_n[1] = ((a[0] ^ a[2]) && (x[0] ^ x[3])) ^ (a[0] && x[2]) ^ a[3] ^
		((a[0] ^ a[2]) && (z[0] ^ z[3])) ^ (a[0] && z[2]) ^ a[3] ^
		((b[0] ^ b[2]) && (y[0] ^ y[3])) ^ (b[0] && y[2]) ^ b[3] ^
		((b[0] ^ b[2]) && (v[0] ^ v[3])) ^ (b[0] && v[2]) ^ b[3] ^
		((c[0] ^ c[2]) && (x[0] ^ x[3])) ^ (c[0] && x[2]) ^ c[3] ^
		((d[0] ^ d[2]) && (y[0] ^ y[3])) ^ (d[0] && y[2]) ^ d[3];

assign o_n[2] =	((a[1] ^ a[3]) && (x[0] ^ x[3])) ^ (a[0] && x[1]) ^ a[3] ^ x[3] ^
		((a[1] ^ a[3]) && (z[0] ^ z[3])) ^ (a[0] && z[1]) ^ a[3] ^ z[3] ^
		((b[1] ^ b[3]) && (y[0] ^ y[3])) ^ (b[0] && y[1]) ^ b[3] ^ y[3] ^
		((b[1] ^ b[3]) && (v[0] ^ v[3])) ^ (b[0] && v[1]) ^ b[3] ^ v[3] ^
		((c[1] ^ c[3]) && (x[0] ^ x[3])) ^ (c[0] && x[1]) ^ c[3] ^ x[3] ^
		((d[1] ^ d[3]) && (y[0] ^ y[3])) ^ (d[0] && y[1]) ^ d[3] ^ y[3];

assign o_o[0] =	((a[1] ^ a[2] ^ a[3]) && (y[1] ^ y[2])) ^ y[3] ^ 
		((b[1] ^ b[2] ^ b[3]) && (x[1] ^ x[2])) ^ x[3] ^ 
		((b[1] ^ b[2] ^ b[3]) && (y[1] ^ y[2])) ^ y[3] ^ 
		((c[1] ^ c[2] ^ c[3]) && (z[1] ^ z[2])) ^ z[3] ^ 
		((c[1] ^ c[2] ^ c[3]) && (v[1] ^ v[2])) ^ v[3] ^ 
		((d[1] ^ d[2] ^ d[3]) && (z[1] ^ z[2])) ^ z[3];

assign o_o[1] =	((a[0] ^ a[2]) && (y[0] ^ y[3])) ^ (a[0] && y[2]) ^ a[3] ^
		((b[0] ^ b[2]) && (x[0] ^ x[3])) ^ (b[0] && x[2]) ^ b[3] ^
		((b[0] ^ b[2]) && (y[0] ^ y[3])) ^ (b[0] && y[2]) ^ b[3] ^
		((c[0] ^ c[2]) && (z[0] ^ z[3])) ^ (c[0] && z[2]) ^ c[3] ^
		((c[0] ^ c[2]) && (v[0] ^ v[3])) ^ (c[0] && v[2]) ^ c[3] ^
		((d[0] ^ d[2]) && (z[0] ^ z[3])) ^ (d[0] && z[2]) ^ d[3];

assign o_o[2] =	((a[1] ^ a[3]) && (y[0] ^ y[3])) ^ (a[0] && y[1]) ^ a[3] ^ y[3] ^
		((b[1] ^ b[3]) && (x[0] ^ x[3])) ^ (b[0] && x[1]) ^ b[3] ^ x[3] ^
		((b[1] ^ b[3]) && (y[0] ^ y[3])) ^ (b[0] && y[1]) ^ b[3] ^ y[3] ^
		((c[1] ^ c[3]) && (z[0] ^ z[3])) ^ (c[0] && z[1]) ^ c[3] ^ z[3] ^
		((c[1] ^ c[3]) && (v[0] ^ v[3])) ^ (c[0] && v[1]) ^ c[3] ^ v[3] ^
		((d[1] ^ d[3]) && (z[0] ^ z[3])) ^ (d[0] && z[1]) ^ d[3] ^ z[3];

assign o_p[0] =	((a[1] ^ a[2] ^ a[3]) && (x[1] ^ x[2])) ^ x[3] ^ 
		((a[1] ^ a[2] ^ a[3]) && (y[1] ^ y[2])) ^ y[3] ^ 
		((b[1] ^ b[2] ^ b[3]) && (x[1] ^ x[2])) ^ x[3] ^ 
		((c[1] ^ c[2] ^ c[3]) && (z[1] ^ z[2])) ^ z[3] ^ 
		((d[1] ^ d[2] ^ d[3]) && (v[1] ^ v[2])) ^ v[3];

assign o_p[1] =	((a[0] ^ a[2]) && (x[0] ^ x[3])) ^ (a[0] && x[2]) ^ a[3] ^
		((a[0] ^ a[2]) && (y[0] ^ y[3])) ^ (a[0] && y[2]) ^ a[3] ^
		((b[0] ^ b[2]) && (x[0] ^ x[3])) ^ (b[0] && x[2]) ^ b[3] ^
		((c[0] ^ c[2]) && (z[0] ^ z[3])) ^ (c[0] && z[2]) ^ c[3] ^
		((d[0] ^ d[2]) && (v[0] ^ v[3])) ^ (d[0] && v[2]) ^ d[3];

assign o_p[2] =	((a[1] ^ a[3]) && (x[0] ^ x[3])) ^ (a[0] && x[1]) ^ a[3] ^ x[3] ^
		((a[1] ^ a[3]) && (y[0] ^ y[3])) ^ (a[0] && y[1]) ^ a[3] ^ y[3] ^
		((b[1] ^ b[3]) && (x[0] ^ x[3])) ^ (b[0] && x[1]) ^ b[3] ^ x[3] ^
		((c[1] ^ c[3]) && (z[0] ^ z[3])) ^ (c[0] && z[1]) ^ c[3] ^ z[3] ^
		((d[1] ^ d[3]) && (v[0] ^ v[3])) ^ (d[0] && v[1]) ^ d[3] ^ v[3];
endmodule

