// Top-module of 8-bit S-box, 
// Sharing: 4-in, 3-out

module gf28inv_shared_poly(
	input i_clk,
	input i_rst_n,
	input [7:0] i_share_1,
	input [7:0] i_share_2,
	input [7:0] i_share_3,
	input [7:0] i_share_4,

	input [3:0] r_mask_1,
	input [3:0] r_mask_2,
	input [3:0] r_mask_3,
	input [3:0] r_mask_4,
	input [3:0] r_mask_5,
	
	output [7:0] o_share_1,
	output [7:0] o_share_2,
	output [7:0] o_share_3
);

logic [59:0] pipe_reg_1;
logic [55:0] pipe_reg_2;

logic [7:0] lmap_1;
logic [7:0] lmap_2;
logic [7:0] lmap_3;
logic [7:0] lmap_4;

logic [3:0] sqsc_1;
logic [3:0] sqsc_2;
logic [3:0] sqsc_3;

logic [3:0] mul_11;
logic [3:0] mul_12;
logic [3:0] mul_13;

logic [3:0] mul_21;
logic [3:0] mul_22;
logic [3:0] mul_23;

logic [3:0] mul_31;
logic [3:0] mul_32;
logic [3:0] mul_33;

logic [3:0] inv_1;
logic [3:0] inv_2;
logic [3:0] inv_3;
logic [3:0] inv_4;
logic [3:0] inv_5;

linmap_shared_poly lm(
	.i_share_1(i_share_1),
	.i_share_2(i_share_2),
	.i_share_3(i_share_3), 
	.i_share_4(i_share_4),

	.o_share_1(lmap_1),
	.o_share_2(lmap_2),
	.o_share_3(lmap_3),
	.o_share_4(lmap_4)
);

invlinmap_shared_poly ilm(
	.i_share_1({mul_21, mul_31}),
	.i_share_2({mul_22, mul_32}),
	.i_share_3({mul_23, mul_33}),

	.o_share_1(o_share_1),
	.o_share_2(o_share_2),
	.o_share_3(o_share_3)
);

gf24mul_shared_poly mul_1(
	.i_data_1(lmap_1[7:4] ^ lmap_1[3:0]), 
	.i_data_2(lmap_2[7:4] ^ lmap_2[3:0]),
	.i_data_3(lmap_3[7:4] ^ lmap_3[3:0]), 
	.i_data_4(lmap_4[7:4] ^ lmap_4[3:0]),
	.i_data_5(lmap_1[3:0]), 
	.i_data_6(lmap_2[3:0]),
	.i_data_7(lmap_3[3:0]),
	.i_data_8(lmap_4[3:0]),

	.o_data_1(mul_11),
	.o_data_2(mul_12),
	.o_data_3(mul_13)
);

gf24mul_shared_poly mul_2(
	.i_data_1(pipe_reg_2[55:52]), 
	.i_data_2(pipe_reg_2[51:48]),
	.i_data_3(pipe_reg_2[47:44]), 
	.i_data_4(pipe_reg_2[43:40]),
	.i_data_5(pipe_reg_2[39:36] ^ pipe_reg_2[35:32]), 
	.i_data_6(pipe_reg_2[31:28]),
	.i_data_7(pipe_reg_2[27:24]), 
	.i_data_8(pipe_reg_2[23:20] ^ pipe_reg_2[19:16]),

	.o_data_1(mul_21),
	.o_data_2(mul_22),
	.o_data_3(mul_23)
);

gf24mul_shared_poly mul_3(
	.i_data_1(pipe_reg_2[39:36] ^ pipe_reg_2[35:32]), 
	.i_data_2(pipe_reg_2[31:28]),
	.i_data_3(pipe_reg_2[27:24]), 
	.i_data_4(pipe_reg_2[23:20] ^ pipe_reg_2[19:16]),
	.i_data_5(pipe_reg_2[15:12]), 
	.i_data_6(pipe_reg_2[11:8]),
	.i_data_7(pipe_reg_2[7:4]), 
	.i_data_8(pipe_reg_2[3:0]),

	.o_data_1(mul_31),
	.o_data_2(mul_32),
	.o_data_3(mul_33)
);

gf24inv_shared_poly inv(
	.i_data_1(pipe_reg_1[39:36] ^ pipe_reg_1[59:56]),
	.i_data_2(pipe_reg_1[35:32]),
	.i_data_3(pipe_reg_1[31:28] ^ pipe_reg_1[27:24]),
	.i_data_4(pipe_reg_1[23:20]),
	.i_data_5(pipe_reg_1[19:16]),

	.o_data_1(inv_1),
	.o_data_2(inv_2),
	.o_data_3(inv_3),
	.o_data_4(inv_4),
	.o_data_5(inv_5)
);

gf24sqsc_shared_poly sqsc(
	.i_share_1(lmap_4[7:4] ^ lmap_3[7:4]),
	.i_share_2(lmap_2[7:4]),
	.i_share_3(lmap_1[7:4]),

	.o_share_1(sqsc_1),
	.o_share_2(sqsc_2),
	.o_share_3(sqsc_3)
);

// Pipeline control
always_ff @(negedge i_clk or negedge i_rst_n) begin
	if (!i_rst_n) begin
		pipe_reg_1 <= '{default: 0};
		pipe_reg_2 <= '{default: 0};
	end
	else begin
		// Set Pipeline Register 1
		pipe_reg_1[59:56] <= sqsc_3;
		pipe_reg_1[55:52] <= lmap_1[7:4];
		pipe_reg_1[51:48] <= lmap_2[7:4];
		pipe_reg_1[47:44] <= lmap_3[7:4];
		pipe_reg_1[43:40] <= lmap_4[7:4];

		pipe_reg_1[39:36] <= sqsc_1 ^ r_mask_1;
		pipe_reg_1[35:32] <= sqsc_2 ^ r_mask_2; 
		pipe_reg_1[31:28] <= r_mask_1 ^ r_mask_2;
		pipe_reg_1[27:24] <= mul_11;
		pipe_reg_1[23:20] <= mul_12;
		pipe_reg_1[19:16] <= mul_13;

		pipe_reg_1[15:12] <= lmap_1[3:0] ^ lmap_1[7:4];
		pipe_reg_1[11:8]  <= lmap_2[3:0] ^ lmap_2[7:4];
		pipe_reg_1[7:4]   <= lmap_3[3:0] ^ lmap_3[7:4];
		pipe_reg_1[3:0]   <= lmap_4[3:0] ^ lmap_4[7:4];
	
		// Set Pipeline Register 2
		pipe_reg_2[55:52] <= pipe_reg_1[55:52];
		pipe_reg_2[51:48] <= pipe_reg_1[51:48];
		pipe_reg_2[47:44] <= pipe_reg_1[47:44];
		pipe_reg_2[43:40] <= pipe_reg_1[43:40];
		
		pipe_reg_2[39:36] <= inv_1;
		pipe_reg_2[35:32] <= inv_2 ^ r_mask_3;
		pipe_reg_2[31:28] <= inv_3 ^ r_mask_4;
		pipe_reg_2[27:24] <= inv_4 ^ r_mask_5;
		pipe_reg_2[23:20] <= inv_5;
		pipe_reg_2[19:16] <= r_mask_3 ^ r_mask_4 ^ r_mask_5;

		pipe_reg_2[15:12] <= pipe_reg_1[15:12];
		pipe_reg_2[11:8]  <= pipe_reg_1[11:8];
		pipe_reg_2[7:4]   <= pipe_reg_1[7:4];
		pipe_reg_2[3:0]   <= pipe_reg_1[3:0];
	end
end


endmodule
