module kex_rsubrcon(
	input [0:3][7:0] i_data,
	input i_mux_ctrl, 
	input [3:0] i_round,
	output [0:3][7:0] o_data
);

logic [0:3][7:0] i_sb;
logic [0:3][7:0] o_sb;
logic [0:3][7:0] i_rc;
logic [0:3][7:0] o_rc;
logic [0:3][7:0] i_rw;
logic [0:3][7:0] o_rw;

logic [0:3][7:0]data_in_t; 
logic [0:3][7:0]data_out_t;

logic op_mode;

assign data_in_t = i_data;
assign o_data = data_out_t;

kex_rotword rw(
	.i_data(i_rw),
	.o_data(o_rw)
);

kex_sbox32 sb32(
	.i_data(i_sb),
	.o_data(o_sb)
);

kex_rcon rc(
	.i_data(i_rc),
	.i_round(i_round),
	.o_data(o_rc)
);

always_comb 
begin
	unique case (i_mux_ctrl)
		1'b0: begin // Sub only
			i_rw = '{default: 0};
			i_sb = data_in_t;			
			i_rc = '{default: 0};
			data_out_t = o_sb;
		end
		default: begin // RSR
			i_rw = data_in_t;
			i_sb = o_rw;
			i_rc = o_sb;
			data_out_t = o_rc;
		end 
	endcase
end	
endmodule
