module key_expansion(
	input i_clk,
	input i_rst_n,
	input [255:0] i_key,
	input [1:0]i_key_rdy,
	output logic o_send_key,
	output logic [0:3][0:3][7:0]o_rkey,
	output logic [3:0] o_current_rnd,
	output logic [1:0]o_key_rdy,
	output logic o_key_stable
);


logic [0:7][0:3][7:0] roundkey_current;
logic [0:7][0:3][7:0] roundkey_next;
logic [0:7][0:3][7:0] roundkey_t;
logic counter_enable;

logic [0:3] b_counter_current;
logic [0:3] r_counter_current;
logic [0:3] k_counter_current;
logic [0:3] l_counter_current;

logic [0:3] b_counter_next;
logic [0:3] r_counter_next;
logic [0:3] k_counter_next;
logic [0:3] l_counter_next;

logic [0:3] b_counter_last;
logic [0:3] k_counter_last;
logic [0:3] l_counter_last;

logic [1:0] mode;
logic expand_key;

// KXOR0 signals 
logic [0:3][7:0] kxor0_sig1;
logic [0:3][7:0] kxor0_sig2;
logic [0:3][7:0] kxor0_out;

// KXOR1 signals
logic [0:3][7:0] kxor1_sig1;
logic [0:3][7:0] kxor1_sig2;
logic [0:3][7:0] kxor1_out;

// KXOR2 signals 
logic [0:3][7:0] kxor2_sig1;
logic [0:3][7:0] kxor2_sig2;
logic [0:3][7:0] kxor2_out;


// Control mode of kxor module
// 0: Passthrough
// 1: xor
logic kxor1_mode;
logic kxor2_mode;

// KRSR1 signals
logic [0:3][7:0] krsr1_sig1;
logic [0:3][7:0] krsr1_sig2;
logic [0:3][7:0] krsr1_out;


// Control mode of krsr module
// 00: Passthrough
// 01: RSR
// 10: XOR
// 11: SUB
logic [1:0] krsr1_mode;
logic [1:0] kxor0_mode;

logic [3:0] krsr_round;

kex_rsr krsr1(
	.i_mode(krsr1_mode),
	.i_data1(krsr1_sig1),
	.i_data2(krsr1_sig2),
	.i_round(krsr_round),
	.o_data(krsr1_out)
);
kex_xor kxor0(
	.i_data1(kxor0_sig1),
	.i_data2(kxor0_sig2),
	.o_data(kxor0_out)
);
kex_xor kxor1(
	.i_data1(kxor1_sig1),
	.i_data2(kxor1_sig2),
	.o_data(kxor1_out)
);
kex_xor kxor2(
	.i_data1(kxor2_sig1),
	.i_data2(kxor2_sig2),
	.o_data(kxor2_out)
);

assign o_key_stable = ~expand_key;

always_ff @(negedge i_clk or negedge i_rst_n)
if (!i_rst_n) begin
	o_rkey <= '{default: 0};
	o_send_key <= 1'b0;
	o_current_rnd <= '{default: 0};
	o_key_rdy <= '{default: 0};
	//o_key_stable <= 1'b0;
end
else begin
	if (i_key_rdy) begin
		o_key_rdy <= '{default: 0};
		//o_key_stable <= 1'b0;
	end
	else if (k_counter_current == k_counter_last && expand_key) begin
		o_key_rdy <= mode;
		//o_key_stable <= 1'b1;
	end 
	

	if (expand_key && b_counter_current == 4'h0) begin 
		o_send_key <= 1'b1;
		o_rkey <= roundkey_t;
		o_current_rnd <= k_counter_current;
	end
	else begin
		o_send_key <= 1'b0;
		o_rkey <= '{default: 0};
		o_current_rnd <= '{default :0};
	end
end

always_ff @(posedge i_clk or negedge i_rst_n)
if (!i_rst_n) begin
	b_counter_current <= '{default:0};
	r_counter_current <= '{default:0};
	l_counter_current <= '{default:0};
	k_counter_current <= '{default:0};
	expand_key <= 1'b0;
end
else begin
	b_counter_current <= b_counter_next;
	r_counter_current <= r_counter_next;
	l_counter_current <= l_counter_next;
	k_counter_current <= k_counter_next;
	if (i_key_rdy != 2'b00 && !expand_key) begin
		b_counter_current <= '{default:0};
		r_counter_current <= '{default:0};
		l_counter_current <= '{default:0};
		k_counter_current <= '{default:0};
		unique case (i_key_rdy)
		2'b01: begin 
			roundkey_current <= {i_key, 128'h0000_0000_0000_0000};
			
			k_counter_last <= 4'ha;
			l_counter_last <= 4'h3;
		end
		2'b10: begin 
			roundkey_current <= {i_key, 64'h0000_0000}; 
			k_counter_last <= 4'hc;
			l_counter_last <= 4'h5;
		end 
		2'b11: begin 
			roundkey_current <= i_key; 
			k_counter_last <= 4'he;	
			l_counter_last <= 4'h7;
		end
		default: begin 
			roundkey_current <= '{default: 0}; 
			k_counter_last <= '{default: 0};
		end
		endcase
		mode <= i_key_rdy;
		expand_key <= 1'b1;
	end 
	else if (k_counter_current == k_counter_last) begin
		expand_key <= 1'b0;
	end
	if (mode == 2'b01 && expand_key) begin
		unique case (l_counter_current)
			4'h0: roundkey_current[0] <= krsr1_out; 
			4'h1: roundkey_current[1] <= kxor1_out;
			4'h2: roundkey_current[2] <= kxor0_out;
			4'h3: roundkey_current[3] <= kxor2_out;
		endcase
		roundkey_current[4:7] <= roundkey_current[4:7];
	end
	if (mode == 2'b10 && expand_key) begin
		unique case (l_counter_current)
			4'h0: roundkey_current[0] <= krsr1_out;
			4'h1: roundkey_current[1] <= kxor1_out;	
			4'h2: roundkey_current[2] <= kxor0_out;
			4'h3: roundkey_current[3] <= kxor2_out;
			4'h4: roundkey_current[4] <= kxor0_out;
			4'h5: roundkey_current[5] <= kxor2_out;
			default: roundkey_current[6] <= roundkey_current[6];
		endcase
		roundkey_current[6:7] <= roundkey_current[6:7];
	end
	if (mode == 2'b11 && expand_key) begin
		unique case (l_counter_current)
			4'h0:roundkey_current[0] <= krsr1_out;
			4'h1:roundkey_current[1] <= kxor1_out;	
			4'h2:roundkey_current[2] <= kxor0_out;
			4'h3:roundkey_current[3] <= kxor2_out;
			4'h4:roundkey_current[4] <= krsr1_out;
			4'h5:roundkey_current[5] <= kxor1_out;	
			4'h6:roundkey_current[6] <= kxor0_out;
			4'h7:roundkey_current[7] <= kxor2_out;
		endcase
	end
end

assign krsr_round = r_counter_current + 4'h1;

always_comb begin
	unique case (mode)
		2'b01:	begin
			 krsr1_sig1 = roundkey_current[0];
			 krsr1_sig2 = roundkey_current[3];
			 kxor1_sig1 = roundkey_current[1];
			 kxor1_sig2 = roundkey_current[0];
			 kxor0_sig1 = roundkey_current[2];
			 kxor0_sig2 = roundkey_current[1];
			 kxor2_sig1 = roundkey_current[3];
			 kxor2_sig2 = roundkey_current[2];
		end
		2'b10: 	begin
			unique case (l_counter_current)  // block 0 special case
				4'h0: begin
					krsr1_sig1 = roundkey_current[0];
					krsr1_sig2 = roundkey_current[5];
					kxor1_sig1 = '{default: 0};
					kxor1_sig2 = '{default: 0};
					kxor0_sig1 = '{default: 0};
					kxor0_sig2 = '{default: 0};
					kxor2_sig1 = '{default: 0};
					kxor2_sig2 = '{default: 0};
				end
				4'h1: begin
					krsr1_sig1 = '{default: 0};
					krsr1_sig2 = '{default: 0};
					kxor1_sig1 = roundkey_current[1];
					kxor1_sig2 = roundkey_current[0];
					kxor0_sig1 = '{default: 0};
					kxor0_sig2 = '{default: 0};
					kxor2_sig1 = '{default: 0};
					kxor2_sig2 = '{default: 0};
				end
				4'h2: begin 
					krsr1_sig1 = '{default: 0};
					krsr1_sig2 = '{default: 0};
					kxor1_sig1 = '{default: 0};
					kxor1_sig2 = '{default: 0};
					kxor0_sig1 = roundkey_current[2];
					kxor0_sig2 = roundkey_current[1];
					kxor2_sig1 = '{default: 0};
					kxor2_sig2 = '{default: 0};
				end
				4'h3: begin
					krsr1_sig1 = '{default: 0};
					krsr1_sig2 = '{default: 0};
					kxor1_sig1 = '{default: 0};
					kxor1_sig2 = '{default: 0};
					kxor0_sig1 = '{default: 0};
					kxor0_sig2 = '{default: 0};
					kxor2_sig1 = roundkey_current[3];
					kxor2_sig2 = roundkey_current[2];
				end
				4'h4: begin
					krsr1_sig1 = '{default: 0};
					krsr1_sig2 = '{default: 0};
					kxor1_sig1 = '{default: 0};
					kxor1_sig2 = '{default: 0};
					kxor0_sig1 = roundkey_current[4];
					kxor0_sig2 = roundkey_current[3];
					kxor2_sig1 = '{default: 0};
					kxor2_sig2 = '{default: 0};
				end
				4'h5: begin
					krsr1_sig1 = '{default: 0};
					krsr1_sig2 = '{default: 0};
					kxor1_sig1 = '{default: 0};
					kxor1_sig2 = '{default: 0};
					kxor0_sig1 = '{default: 0};
					kxor0_sig2 = '{default: 0};
					kxor2_sig1 = roundkey_current[5];
					kxor2_sig2 = roundkey_current[4];
				end
				default: begin
					krsr1_sig1 = '{default: 0};
					krsr1_sig2 = '{default: 0};
					kxor1_sig1 = '{default: 0};
					kxor1_sig2 = '{default: 0};
					kxor0_sig1 = '{default: 0};
					kxor0_sig2 = '{default: 0};
					kxor2_sig1 = '{default: 0};
					kxor2_sig2 = '{default: 0};

				end
			endcase
		end
		2'b11:	begin
			unique case (l_counter_current)
				4'h0: begin
					krsr1_sig1 = roundkey_current[0];
					krsr1_sig2 = roundkey_current[7];
					kxor1_sig1 = '{default: 0};
					kxor1_sig2 = '{default: 0};
					kxor0_sig1 = '{default: 0};
					kxor0_sig2 = '{default: 0};
					kxor2_sig1 = '{default: 0};
					kxor2_sig2 = '{default: 0};
				end
				4'h1: begin
					krsr1_sig1 = '{default: 0};
					krsr1_sig2 = '{default: 0};
					kxor1_sig1 = roundkey_current[1];
					kxor1_sig2 = roundkey_current[0];
					kxor0_sig1 = '{default: 0};
					kxor0_sig2 = '{default: 0};
					kxor2_sig1 = '{default: 0};
					kxor2_sig2 = '{default: 0};
				end
				4'h2: begin
					krsr1_sig1 = '{default: 0};
					krsr1_sig2 = '{default: 0};
					kxor1_sig1 = '{default: 0};
					kxor1_sig2 = '{default: 0};
					kxor0_sig1 = roundkey_current[2];
					kxor0_sig2 = roundkey_current[1];
					kxor2_sig1 = '{default: 0};
					kxor2_sig2 = '{default: 0};
				end
				4'h3: begin
					krsr1_sig1 = '{default: 0};
					krsr1_sig2 = '{default: 0};
					kxor1_sig1 = '{default: 0};
					kxor1_sig2 = '{default: 0};
					kxor0_sig1 = '{default: 0};
					kxor0_sig2 = '{default: 0};
					kxor2_sig1 = roundkey_current[3];
					kxor2_sig2 = roundkey_current[2];
				end
				4'h4: begin
					krsr1_sig1 = roundkey_current[4];
					krsr1_sig2 = roundkey_current[3];
					kxor1_sig1 = '{default: 0};
					kxor1_sig2 = '{default: 0};
					kxor0_sig1 = '{default: 0};
					kxor0_sig2 = '{default: 0};
					kxor2_sig1 = '{default: 0};
					kxor2_sig2 = '{default: 0};
				end
				4'h5: begin
					krsr1_sig1 = '{default: 0};
					krsr1_sig2 = '{default: 0};
					kxor1_sig1 = roundkey_current[5];
					kxor1_sig2 = roundkey_current[4];
					kxor0_sig1 = '{default: 0};
					kxor0_sig2 = '{default: 0};
					kxor2_sig1 = '{default: 0};
					kxor2_sig2 = '{default: 0};
				end
				4'h6: begin
					krsr1_sig1 = '{default: 0};
					krsr1_sig2 = '{default: 0};
					kxor1_sig1 = '{default: 0};
					kxor1_sig2 = '{default: 0};
					kxor0_sig1 = roundkey_current[6];
					kxor0_sig2 = roundkey_current[5];
					kxor2_sig1 = '{default: 0};
					kxor2_sig2 = '{default: 0};
				end
				4'h7: begin
					krsr1_sig1 = '{default: 0};
					krsr1_sig2 = '{default: 0};
					kxor1_sig1 = '{default: 0};
					kxor1_sig2 = '{default: 0};
					kxor0_sig1 = '{default: 0};
					kxor0_sig2 = '{default: 0};
					kxor2_sig1 = roundkey_current[7];
					kxor2_sig2 = roundkey_current[6];
				end
			endcase 
		end
		default: begin
			 krsr1_sig1 = roundkey_current[0];
			 krsr1_sig2 = roundkey_current[3];
			 kxor1_sig1 = roundkey_current[1];
			 kxor1_sig2 = roundkey_current[0];
			 kxor0_sig1 = roundkey_current[2];
			 kxor0_sig2 = roundkey_current[1];
			 kxor2_sig1 = roundkey_current[3];
			 kxor2_sig2 = roundkey_current[2];
		end
	endcase
end

always_comb begin
	unique case (mode)
		2'b01: begin 
			roundkey_t = roundkey_current[0:3];
		end
		2'b10: begin
			unique case (k_counter_current)
				4'h0, 4'h3, 4'h6, 4'h9, 4'hc: begin
					roundkey_t = roundkey_current[0:3];	
				end
				4'h1, 4'h4, 4'h7, 4'ha, 4'hd: begin
					roundkey_t = {roundkey_current[4:5], roundkey_current[0:1]};
				end
				4'h2, 4'h5, 4'h8, 4'hb: begin
					roundkey_t = roundkey_current[2:5];
				end
				default: begin
					roundkey_t = roundkey_current;
				end
			endcase
		end
		2'b11: begin
			unique case (k_counter_current)
				4'h0, 4'h2, 4'h4, 4'h6, 4'h8, 4'ha, 4'hc, 4'he: begin
					roundkey_t = roundkey_current[0:3];
				end
				4'h1, 4'h3, 4'h5, 4'h7, 4'h9, 4'hb, 4'hd, 4'hf: begin
					roundkey_t = roundkey_current[4:7];
				end
			endcase
		end
		default: begin
			roundkey_t = roundkey_current;
		end
	endcase
end

always_comb begin
	unique case (mode)
		2'b01: begin
				krsr1_mode = 2'b01;
		end
		2'b10: begin
				krsr1_mode = 2'b01;
		end
		2'b11: begin
			unique case (k_counter_current)
				4'h0, 4'h2,4'h4,4'h6,4'h8,4'ha,4'hc,4'he: begin
					krsr1_mode = 2'b01;
				end
				4'h1, 4'h3,4'h5,4'h7,4'h9,4'hb,4'hd,4'hf: begin
					krsr1_mode = 2'b11;
				end
			endcase
		end
		default: begin
			krsr1_mode = 2'b00;
		end
	endcase
end


always_comb begin: counter
	if (expand_key && b_counter_current < 4'h3)
		b_counter_next = b_counter_current + 4'h1;
	else
		b_counter_next = 4'h0;
	if (expand_key && k_counter_current < k_counter_last && b_counter_current == 4'h3) begin
		k_counter_next = k_counter_current + 4'h1;
	end
	else begin
		k_counter_next = k_counter_current;
	end

	if (expand_key && l_counter_current < l_counter_last) begin
		l_counter_next = l_counter_current + 4'h1; 
		r_counter_next = r_counter_current;
	end
	else if (expand_key && k_counter_current < k_counter_last && l_counter_current == l_counter_last) begin
		l_counter_next = 4'h0;
		r_counter_next = r_counter_current + 4'h1;
	end
	else begin
		l_counter_next = l_counter_current;
		r_counter_next = r_counter_current;
		
	end
end: counter

endmodule

module kex_xor(
	input [0:3][7:0] i_data1,
	input [0:3][7:0] i_data2,
	output logic [0:3][7:0] o_data
);

always_comb begin
	o_data = i_data1 ^ i_data2;
end

endmodule

module kex_rsr(
	input [1:0] i_mode,
	input [0:3][7:0] i_data1,
	input [0:3][7:0] i_data2,
	input [0:3] i_round,
	output [0:3][7:0] o_data
);

logic [0:3][7:0] i_sb;
logic [0:3][7:0] o_sb;
logic [0:3][7:0] i_rc;
logic [0:3][7:0] o_rc;
logic [0:3][7:0] i_rw;
logic [0:3][7:0] o_rw;

logic [0:3][7:0]data1_in_t; 
logic [0:3][7:0]data2_in_t; 
logic [0:3][7:0]data_out_t;

logic op_mode;

assign data1_in_t = i_data1;
assign data2_in_t = i_data2;
assign o_data = data_out_t;

kex_rotword rw(
	.i_data(i_rw),
	.o_data(o_rw)
);

kex_sbox32 sb32(
	.i_data(i_sb),
	.o_data(o_sb)
);

kex_rcon rc(
	.i_data(i_rc),
	.i_round(i_round),
	.o_data(o_rc)
);

always_comb 
begin
	unique case (i_mode)
		2'b00: begin 
			i_rw = '{default: 0};
			i_sb = '{default: 0};
			i_rc = '{default: 0};
			data_out_t = data1_in_t;
		end
		2'b01: begin
			i_rw = data2_in_t;
			i_sb = o_rw;
			i_rc = o_sb;
			data_out_t = o_rc ^ data1_in_t;
		end
		2'b10: begin
			i_rw = '{default: 0};
			i_sb = '{default: 0};
			i_rc = '{default: 0};
			data_out_t = data1_in_t ^ data2_in_t;
		end
		2'b11: begin
			i_rw = '{default:0};
			i_sb = data2_in_t;
			i_rc = '{default: 0};
			data_out_t = o_sb ^ data1_in_t;
		end
		default: begin
			i_rw = '{default: 0};
			i_sb = '{default: 0};
			i_rc = '{default: 0};
			data_out_t = data1_in_t;
		end 
	endcase
end	
endmodule
