module key_logic_um(
	input i_clk,
	input i_rst_n,
	input [1:0] i_mode, // encrypt/decrypt + load/store
	input [3:0] i_saddr,
	input [3:0] i_laddr,
	input [1:0] i_flf,
	input [0:3][0:3][7:0] i_rkey,
	output logic [0:3][0:3][7:0] o_m1_key
);

logic [0:14][0:3][0:3][7:0] m1_key;

logic [0:3][0:3][7:0] t_data1;

logic [0:3][0:3][7:0] t_odata1;

always_comb begin
	if (i_mode == 2'b11) begin
		t_data1 = '{default: 0};
		o_m1_key = m1_key[i_laddr];
	end
	else if (i_mode == 2'b01) begin
		if (i_flf == 2'b00) begin
		t_data1 = m1_key[i_laddr];
		o_m1_key = t_odata1;
		end
		else begin
		t_data1 = '{default: 0};
		o_m1_key = m1_key[i_laddr];
		end
	end	
	else begin
		t_data1 = '{default: 0};
		o_m1_key = '{default: 0};
	end
end

always_ff @(posedge i_clk or negedge i_rst_n)
if (!i_rst_n) begin
	m1_key <= '{default: 0};
end
else begin
	unique case (i_mode) // [0] = 0 STORE mode, [0] = 1 LOAD mode
		2'b00: begin
			m1_key[i_saddr] <= i_rkey ;
		end
		2'b10: begin
			m1_key[i_saddr] <= i_rkey;
		end
		2'b11: begin // encrypt fetch key mode
			m1_key[i_laddr] <= m1_key[i_laddr];
		end
		2'b01: begin // decrypt fetch key mode
			m1_key[i_laddr] <= m1_key[i_laddr];
		end
		default: begin
			m1_key[i_laddr] <= m1_key[i_laddr];
		end 	
endcase
	
end

genvar index;  
generate  
for (index=0; index < 4; index=index+1)  
  	begin: gen_imc_blocks1  
		invmixcol imc_inst(
			.i_data(t_data1[index]),
			.i_mode(!i_mode[1]),
			.o_data(t_odata1[index])
		);
	end
endgenerate

endmodule

