
module mixcol_dual(
	input [0:3][7:0] i_data,
	input i_mode,
	output logic [0:3][7:0] o_data
);

logic [0:3][7:0] o_mc;
logic [0:3][7:0] t_data;
logic [7:0] o_x4_02;
logic [7:0] o_x4_13;
logic [7:0] i_xt;
logic [7:0] o_xt;
logic [7:0] shared02;
logic [7:0] shared13;

mixcol mc(
	.i_data(i_data),
	.o_data(o_mc)
);

xtime xt(
	.i_data(i_xt),
	.o_data(o_xt)
);

x4time x4t1(
	.i_data(shared02),
	.o_data(o_x4_02)
);

x4time x4t2(
	.i_data(shared13),
	.o_data(o_x4_13)
);

assign shared02 = i_data[0] ^ i_data[2];
assign shared13 = i_data[1] ^ i_data[3];
assign i_xt = o_x4_02 ^ o_x4_13;

assign t_data[0] = o_mc[0] ^ o_xt ^ o_x4_02;
assign t_data[1] = o_mc[1] ^ o_xt ^ o_x4_13;
assign t_data[2] = o_mc[2] ^ o_xt ^ o_x4_02;
assign t_data[3] = o_mc[3] ^ o_xt ^ o_x4_13;

always_comb 
begin
	// mixcol
	if (i_mode) begin 
		o_data = o_mc;
	end
	// invmixcol
	else begin 	
		o_data = t_data;
	end
end

endmodule
