/* -----------------------------------------------------------------------------------------------
	This testbench generates VCD-data for power analysis and/or simulated power attacks 
 	that can be used with PrimetimePX.
 	Warning: Generates huge VCD-files
----------------------------------------------------------------------------------------------- */ 


module randgen_poly_tb();

logic clk;
logic encrypt;
logic rst;
logic ciphertxt_rdy;
logic plaintxt_rdy;
logic [0:3][0:3][7:0] plaintxt;
logic [0:15][0:4][3:0] rnd_bits;
logic [0:3][0:3][7:0] ciphertext;
logic [0:3][0:3][7:0] sm1;
logic [0:3][0:3][7:0] sm2;
logic [0:3][0:3][7:0] sm3;
logic [0:3][0:3][7:0] km1;
logic [0:3][0:3][7:0] km2;
logic [255:0] key;
logic [1:0] key_rdy;
logic key_ack;
logic cbc_enable;

logic [0:3][0:3][7:0] iv;
logic [0:3][0:3][7:0] state_reg;
logic [0:3][0:3][7:0] sbox_output;
logic [0:3][0:3][7:0] add_roundkey_output;

assign state_reg = aes.share_1.o_data ^ aes.share_2.o_data ^ aes.share_3.o_data;
assign add_roundkey_output = aes.ci.sm_1_xor ^ aes.ci.sm_2_xor ^ aes.ci.sm_3_xor;
assign sbox_output = aes.sb.o_data1 ^ aes.sb.o_data2 ^ aes.sb.o_data3;

aes_core_shared_poly aes(
	.i_clk(clk),
	.i_rst_n(rst),
	.i_encrypt(encrypt), 		
	.i_data(plaintxt),
	.i_data_rdy(plaintxt_rdy),
	.i_cbc_enable(cbc_enable),

	.i_sm_1(sm1),
	.i_sm_2(sm2),
	.i_sm_3(sm3),
	.i_km_1(km1),
	.i_km_2(km2),

	.i_iv(iv),

	.i_key(key),
	.i_key_rdy(key_rdy),
	.i_rm_bits(rnd_bits),
	.o_key_ack(key_ack),
	.o_data_rdy(ciphertxt_rdy),
	.o_data(ciphertext)
);

int count = 1000;
class random_input;
	rand bit [127:0] plaintxt;
	rand bit [127:0] s1;
	rand bit [127:0] s2;
	rand bit [127:0] s3;
	rand bit [127:0] k1;
	rand bit [127:0] k2;
	rand bit [0:15][0:4][3:0] rbits;
endclass
static random_input idata = new();

class random_key_iv;
	rand bit [255:0] keydata;
	rand bit [127:0] ivdata;
endclass
static random_key_iv key_iv = new();

always #1 clk = !clk;
always #1 begin
	idata.randomize();
	sm1 = idata.s1;
	sm2 = idata.s2;
	sm3 = idata.s3;
	km1 = idata.k1;
	km2 = idata.k2;
	rnd_bits = idata.rbits;
end


initial 
begin
	$dumpfile("response.vcd");
	$dumpvars(0);
	$dumpon;
	key_iv.randomize();
	clk = 0;
	rst = 0;
	cbc_enable = 1'b1;
	plaintxt_rdy = 1'b0;
	key_rdy = 2'b00;

	#2
	rst = 1;
	key_rdy = 2'b10;
	encrypt = 1'b1;

	key = key_iv.keydata;
	iv = key_iv.ivdata;
	#2
	key_rdy = 2'b00;
	wait(key_ack);
	$display("Key expansion finished");
	repeat(count) begin
		#7
		idata.randomize();
		plaintxt = idata.plaintxt;
		plaintxt_rdy = 1'b1;
		#6
		plaintxt_rdy = 1'b0;
		wait(ciphertxt_rdy)
		$display("Encryption finished");
	end
	$display("Simulation End");
	$dumpoff;
	$stop;
end

endmodule

