module round_counter(
	input i_clk,
	input i_rst_n,
	input i_count_enable,
	input i_encrypt,
	input [1:0] i_mode,
	output logic [1:0] o_flf,  // [00 01 11 10] middle, first, last or finished round
	output logic [3:0] o_round
);

logic [3:0] current_round;
logic running;
logic [3:0] last_round;
logic [3:0] next_round;
logic [3:0] round;
logic [1:0] flf;


always_ff @(posedge i_clk or negedge i_rst_n) 
if (!i_rst_n) begin
	current_round <= '{default: 0};
end
else begin
	o_flf <= flf;
	current_round <= next_round;
end

always_comb 
begin
	if ((current_round < last_round) && i_count_enable) begin
		next_round = current_round + 1;
	end
	else begin
		next_round = '{default :0};
	end
end

always_comb
begin
	unique if (next_round == 4'h0)
		flf = 2'b01;
	else if (next_round == last_round)
		flf = 2'b11;
	else
		flf = 2'b00;
end

always_comb
begin
	unique case (i_mode)
		2'h1:	last_round = 4'ha;
		2'h2:	last_round = 4'hc;
		default:last_round = 4'he;
	endcase	
end

always_comb
begin
	unique case (i_encrypt)
		1'b1: o_round = current_round;
		default: o_round = last_round - current_round;
	endcase
end

endmodule

/*
logic [3:0] rcounter;
logic [3:0] maxcount;
logic [1:0] flf;
logic enabled;

assign o_flf = flf;
//assign enabled = i_count_enable;
assign o_rnd_exec = enabled;

always_comb begin
	enabled = i_count_enable; 
	unique case (i_encrypt) 
		1'b0: unique case (i_mode) 
			2'h1: o_round = 4'ha - rcounter;
			2'h2: o_round = 4'hc - rcounter;
			2'h3: o_round = 4'he - rcounter;
			default: o_round = 4'h0;
			endcase
		default:  o_round = rcounter;
	endcase
end

always_ff @(posedge i_clk or negedge i_rst_n) 
if (!i_rst_n) begin
	rcounter <= '{default: 0};
end
else begin

	//o_round <= rcounter;
	if ((rcounter < maxcount) && enabled) begin
		rcounter <= rcounter + 1;
	end
	else begin
		rcounter <= '{default: 0};
	end
end

always_comb 
begin
	unique case (rcounter)
		4'h0: begin
			unique case (i_mode)
				2'h1: maxcount = 4'ha;
				2'h2: maxcount = 4'hc;
				2'h3: maxcount = 4'he;
				default: maxcount = 4'h0;
			endcase
			flf = 2'b01;
		end
		4'h1,4'h2,4'h3,4'h4,4'h5,4'h6,4'h7,4'h8: begin
			flf = 2'b00;
		end
		4'h9: begin 
		if (maxcount == 4'ha)
			flf = 2'b00;
		   else 
			flf = 2'b00;
		end
		4'ha: if (maxcount == 4'ha) begin
			flf = 2'b11;
			end
		    else begin
			flf = 2'b00;
			end
		4'hb: begin
		if (maxcount == 4'hc)
			flf = 2'b00;
		    else 
			flf = 2'b00;
		end
		4'hc: if (maxcount == 4'hc) begin
			flf = 2'b11;
			end
		    else begin 
			flf = 2'b00;
			end
		4'hd: begin
		if (maxcount == 4'he)
			flf = 2'b00;
	 	    else 
			flf = 2'b00;
		end
		4'he: if (maxcount == 4'he) begin
			flf = 2'b11;
			end
		    else begin
			flf = 2'b00;
			end
		default: begin
			flf = 2'b00;
		end
	endcase
		
end

endmodule
*/
