module sbox128_shared_norm(
	input i_clk,
	input i_rst_n,
	input i_encrypt,
	input [0:3][0:3][7:0] i_data1,
	input [0:3][0:3][7:0] i_data2,
	input [0:3][0:3][7:0] i_data3,
	input [0:3][0:3][7:0] i_data4,

	input [0:3][0:3][0:4][3:0] i_mask,

	output [0:3][0:3][7:0] o_data1,
	output [0:3][0:3][7:0] o_data2,
	output [0:3][0:3][7:0] o_data3

);

logic [0:3][0:3][7:0] i_sb1;
logic [0:3][0:3][7:0] i_sb2;
logic [0:3][0:3][7:0] i_sb3;
logic [0:3][0:3][7:0] i_sb4;
logic [0:3][0:3][7:0] o_sb1;
logic [0:3][0:3][7:0] o_sb2;
logic [0:3][0:3][7:0] o_sb3;

genvar index1;  
genvar index2;  
generate  
for (index1=0; index1 < 4; index1 = index1 + 1)  begin
	for (index2=0; index2 < 4; index2=index2+1)  
  	begin: gen_blocks_blocks  
    	aft_shared aft_inst (  
      		.i_enable(i_encrypt), 
      		.i_data_1(o_sb1[index1][index2]),
		.i_data_2(o_sb2[index1][index2]),
		.i_data_3(o_sb3[index1][index2]),
		.o_data_1(o_data1[index1][index2]),	
		.o_data_2(o_data2[index1][index2]),
		.o_data_3(o_data3[index1][index2])
	);  

	aft_inv_shared afti_inst(
		.i_enable(!i_encrypt),	
		.i_data_1(i_data1[index1][index2]),
		.i_data_2(i_data2[index1][index2]),
		.i_data_3(i_data3[index1][index2]),
		.i_data_4(i_data4[index1][index2]),
		.o_data_1(i_sb1[index1][index2]),
		.o_data_2(i_sb2[index1][index2]),
		.o_data_3(i_sb3[index1][index2]),
		.o_data_4(i_sb4[index1][index2])
	);

	gf28inv_shared_norm sb_inst(  // Normal basis S-box
		.i_clk(i_clk),
		.i_rst_n(i_rst_n),
		.i_share_1(i_sb1[index1][index2]),
		.i_share_2(i_sb2[index1][index2]),
		.i_share_3(i_sb3[index1][index2]),
		.i_share_4(i_sb4[index1][index2]),
		
		.r_mask_1(i_mask[index1][index2][0]),
		.r_mask_2(i_mask[index1][index2][1]),
		.r_mask_3(i_mask[index1][index2][2]),
		.r_mask_4(i_mask[index1][index2][3]),
		.r_mask_5(i_mask[index1][index2][4]),
			
		.o_share_1(o_sb1[index1][index2]),
		.o_share_2(o_sb2[index1][index2]),
		.o_share_3(o_sb3[index1][index2])
	);
  end  
end
endgenerate  

endmodule
