module scbc(
	input i_clk,
	input i_rst_n,
	input i_encrypt,
	input i_key_rdy_kex,
	input i_data_rdy,
	input [1:0] i_flf,
	input i_cbc_enable,
	
	input [0:3][0:3][7:0] i_data_iv,
	input [0:3][0:3][7:0] i_out_cipher,
	input [0:3][0:3][7:0] i_in_cipher,

	output logic [0:3][0:3][7:0] o_encrypt,
	output logic [0:3][0:3][7:0] o_decrypt
);

logic [0:3][0:3][7:0] internal_reg1;
logic [0:3][0:3][7:0] internal_reg2;
logic [1:0] started;

always_comb 
begin
	if(!i_cbc_enable) begin
		o_encrypt = '{default: 0};
		o_decrypt = '{default: 0};
	end
	else
	case (i_encrypt)
		// encrypt mode
		1'b1: begin
			o_encrypt = internal_reg1;
			o_decrypt = '{default: 0};
		end
		// decrypt mode
		default: begin
			o_decrypt = internal_reg2;
			o_encrypt = '{default: 0}; 
		end
	endcase
end


always_ff @(posedge i_clk or negedge i_rst_n) begin
if (!i_rst_n) begin
	internal_reg1 <= '{default: 0};
	internal_reg2 <= '{default: 0};
	started <= 2'b00;
end
else begin
	if ((i_key_rdy_kex) && !started) begin
		internal_reg1 <= i_data_iv;	
		internal_reg2 <= '{default: 0};
		started <= 2'b11;
	end
	else if (i_encrypt && i_data_rdy && started[0]) begin
		internal_reg1 <= internal_reg1;
		internal_reg2 <= '{default: 0};
		started <= 2'b10;
	end
	else if (i_encrypt && i_flf[1]) begin
		internal_reg1 <= i_out_cipher;
		internal_reg2 <= '{default: 0};
		started <= 2'b10;
	end
	else if (!i_encrypt && i_data_rdy && started[0]) begin	
		internal_reg2 <= internal_reg1;
		internal_reg1 <= i_in_cipher;
		started <= 2'b10;
	end
	else if (!i_encrypt && i_flf[1]) begin
		internal_reg2 <= internal_reg2;
		internal_reg1 <= internal_reg1;
		started <= 2'b11;		
	end	
end	
end
endmodule
