// combined ShiftRows & MixCol-module

module sr_mc(
	input [0:3][0:3][7:0] i_data,
	input [1:0] i_mode,
	output logic [0:3][0:3][7:0] o_data

);

logic [0:3][0:3][7:0] t_shifted;
logic [0:3][0:3][7:0] mc;
logic [0:3][0:3][7:0] o_mc;

mixcol_dual mc1(
	.i_data(mc[0]),
	.i_mode(i_mode[1]),
	.o_data(o_mc[0])
);

mixcol_dual mc2(
	.i_data(mc[1]),
	.i_mode(i_mode[1]),
	.o_data(o_mc[1])
);

mixcol_dual mc3(
	.i_data(mc[2]),
	.i_mode(i_mode[1]),
	.o_data(o_mc[2])
);

mixcol_dual mc4(
	.i_data(mc[3]),
	.i_mode(i_mode[1]),
	.o_data(o_mc[3])
);


assign mc = t_shifted;

always_comb begin
	unique case (i_mode) 
		// ENCRYPT_MODE
		2'b10:	begin
			t_shifted[0][0] = i_data[0][0];
			t_shifted[1][0] = i_data[1][0];
			t_shifted[2][0] = i_data[2][0];
			t_shifted[3][0] = i_data[3][0];

			t_shifted[0][1] = i_data[1][1];
			t_shifted[1][1] = i_data[2][1];
			t_shifted[2][1] = i_data[3][1];
			t_shifted[3][1] = i_data[0][1];

			t_shifted[0][2] = i_data[2][2];
			t_shifted[1][2] = i_data[3][2];
			t_shifted[2][2] = i_data[0][2];
			t_shifted[3][2] = i_data[1][2];

			t_shifted[0][3] = i_data[3][3];
			t_shifted[1][3] = i_data[0][3];
			t_shifted[2][3] = i_data[1][3];
			t_shifted[3][3] = i_data[2][3];

			o_data = o_mc;
			end
		// DECRYPT_MODE
		2'b00:	begin
			t_shifted[0][0] = i_data[0][0];
			t_shifted[1][0] = i_data[1][0];
			t_shifted[2][0] = i_data[2][0];
			t_shifted[3][0] = i_data[3][0];

			t_shifted[0][1] = i_data[3][1];
			t_shifted[1][1] = i_data[0][1];
			t_shifted[2][1] = i_data[1][1];
			t_shifted[3][1] = i_data[2][1];

			t_shifted[0][2] = i_data[2][2];
			t_shifted[1][2] = i_data[3][2];
			t_shifted[2][2] = i_data[0][2];
			t_shifted[3][2] = i_data[1][2];

			t_shifted[0][3] = i_data[1][3];
			t_shifted[1][3] = i_data[2][3];
			t_shifted[2][3] = i_data[3][3];
			t_shifted[3][3] = i_data[0][3];

			o_data = o_mc;
			end
		// LAST_RND_ENC_MODE
		2'b11: 	begin

			t_shifted[0][0] = i_data[0][0];
			t_shifted[1][0] = i_data[1][0];
			t_shifted[2][0] = i_data[2][0];
			t_shifted[3][0] = i_data[3][0];

			t_shifted[0][1] = i_data[1][1];
			t_shifted[1][1] = i_data[2][1];
			t_shifted[2][1] = i_data[3][1];
			t_shifted[3][1] = i_data[0][1];

			t_shifted[0][2] = i_data[2][2];
			t_shifted[1][2] = i_data[3][2];
			t_shifted[2][2] = i_data[0][2];
			t_shifted[3][2] = i_data[1][2];

			t_shifted[0][3] = i_data[3][3];
			t_shifted[1][3] = i_data[0][3];
			t_shifted[2][3] = i_data[1][3];
			t_shifted[3][3] = i_data[2][3];

			o_data = t_shifted;
			end
		// LAST_RND_DEC_MODE
		default: begin
			t_shifted[0][0] = i_data[0][0];
			t_shifted[1][0] = i_data[1][0];
			t_shifted[2][0] = i_data[2][0];
			t_shifted[3][0] = i_data[3][0];

			t_shifted[0][1] = i_data[3][1];
			t_shifted[1][1] = i_data[0][1];
			t_shifted[2][1] = i_data[1][1];
			t_shifted[3][1] = i_data[2][1];

			t_shifted[0][2] = i_data[2][2];
			t_shifted[1][2] = i_data[3][2];
			t_shifted[2][2] = i_data[0][2];
			t_shifted[3][2] = i_data[1][2];

			t_shifted[0][3] = i_data[1][3];
			t_shifted[1][3] = i_data[2][3];
			t_shifted[2][3] = i_data[3][3];
			t_shifted[3][3] = i_data[0][3];

			o_data = t_shifted;
			end
	endcase
end



endmodule
